/*
 * irp_instance.h
 *
 *  Created on: 26 de jul de 2016
 *      Author: hokama
 */

#ifndef IRP2D_INSTANCE_H_
#define IRP2D_INSTANCE_H_

#include <iostream>
#include <fstream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>
#include <libgen.h>

using namespace std;

namespace IRP2D{
class Irp2dInstance{

private:
  //instance parameters
  int number_of_customers, number_of_products, number_of_vehicles, horizon;

  //customer parameters
  vector < vector <vector < int > > > demand_of_product_per_customer_per_period;
  vector < int > max_area_customer;
  vector < int > x;
  vector < int > y;

  //bin parameters
  int W;
  int H;

  //items parameters
  vector < int > width;
  vector < int > height;

  //auxiliary data
  vector <vector < int > > distance;
  vector < vector < int > > edge_id;
  string instance_type;
  string file_name;
  string file_base_name;
  string output_path;

protected:
  int verbose;

public:
  vector < bool > feasible_routes;
  vector < vector < pair <int, int> > > routes;
  vector < vector < int > > x_routes;
  vector < vector < int > > y_routes;
  vector < vector < int > > w_routes;
  vector < vector < int > > h_routes;
  vector < vector < int > > p_routes;

	Irp2dInstance();
	virtual ~Irp2dInstance();

  void readInstance(string input_name) {
    ifstream input(input_name.c_str());
    set_file_name(input_name);
    if (input.is_open()){
      input >> *this;
      input.close();
    }else{
      cerr << "File not found!";
      exit(-1);
    }
    return;
  }

  friend std::istream &operator>>( std::istream &, Irp2dInstance &);
  friend std::ostream &operator<<( std::ostream &, Irp2dInstance &);

  //getters and setters
  int get_number_of_customers(){return number_of_customers;};
  int get_number_of_products(){return number_of_products;};
  int get_number_of_vehicles(){return number_of_vehicles;};
  void set_number_of_vehicles(int k){number_of_vehicles = k;};
  void set_max_area_customer(int i, int a){max_area_customer[i] = a; return;}
  int get_max_area_customer(int i){return max_area_customer[i];}

  int get_horizon(){return horizon;};
  int get_distance(int i, int j){return distance[i][j];};
  int get_x(int i){return x[i];};
  int get_y(int i){return y[i];};

  int get_width(int p){return width[p];};
  int get_height(int p){return height[p];};

  int get_vehicle_width(){return W;};
  int get_vehicle_height(){return H;};

  int get_demand_of_product_per_customer_per_period(int customer, int product, int period){return demand_of_product_per_customer_per_period[customer][product][period];};

  int get_edge_id(int i, int j){return edge_id[i][j];};

  string get_instance_type(){return instance_type;};
  void set_instance_type(string t){instance_type.clear(); instance_type.append(t);};
  void set_output_path(string op){output_path = op; return;}
  void set_file_name(string fn){
    file_name = fn;
    file_base_name = basename((char *) file_name.c_str());
    return;}
  void set_verbose(int v){verbose = v;}
  int get_verbose(){return verbose ;}
  string get_file_name(){return file_name;}
  string get_file_base_name(){return file_base_name;}
  string get_output_path(){return output_path;}
};
} // end namespace

#endif /* IRP2D_INSTANCE_H_ */
